"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TestServer_1 = require("../TestServer");
const services = require("../services");
const Token_1 = require("../shared/Token");
const chai_1 = require("chai");
const chai = require("chai");
chai.use(require("chai-as-promised"));
const superagent = require("superagent");
const uuid = require("uuid");
describe('User Metadata', function () {
    let server;
    let usernamePasswordAuthService;
    let adminUser;
    before(() => __awaiter(this, void 0, void 0, function* () {
        server = new TestServer_1.TestServer();
        usernamePasswordAuthService = new services.UsernamePasswordAuthService();
        server.addService(usernamePasswordAuthService);
        server.addService(new services.RealmDirectoryService());
        server.addService(new services.SyncProxyService());
        server.addService(new services.SyncService());
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        server.shutdown();
    }));
    describe("get", () => {
        describe("programatically", () => {
            it('should be able to get a user', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const ORIGINALPASSWORD = 'original';
                    yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
                    const fetchedUser = usernamePasswordAuthService.getUserByProviderId(USERNAME, "password");
                    chai_1.expect(fetchedUser).not.to.be.undefined;
                    chai_1.expect(fetchedUser.accounts[0].providerId).to.be.eq(USERNAME);
                });
            });
        });
        describe("via REST API", () => {
            it("without a token it should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(superagent.get(`${server.url}/auth/users/someuser`), "Forbidden");
            }));
            it("without an admin token it should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, "somepassword", false);
                const token = new Token_1.RefreshToken({ identity: "some user", isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isRejected(superagent.get(`${server.url}/auth/users/${user.userId}`), "Forbidden");
            }));
            it("with an admin token it should get a user", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, "somepassword", false);
                const token = new Token_1.RefreshToken({ identity: "some user", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isFulfilled(superagent.get(`${server.url}/auth/users/${user.userId}`)
                    .set({
                    Authorization: token,
                }));
            }));
            it("with the user's token it should get a user", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, "somepassword", false);
                const token = new Token_1.RefreshToken({ identity: user.userId, isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isFulfilled(superagent.get(`${server.url}/auth/users/${user.userId}`)
                    .set({
                    Authorization: token,
                }));
            }));
        });
    });
    it('should be able get a user via REST API with providerId', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'original';
            const EMAIL = `${USERNAME}@realm.io`;
            const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
            const token = new Token_1.RefreshToken({ identity: user.userId, isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
            const response = yield superagent.get(`${server.url}/auth/users/password/${USERNAME}`)
                .set({
                Authorization: token,
            });
            chai_1.expect(response.body).not.to.be.undefined;
            chai_1.expect(response.body["accounts"]["0"].providerId).to.eq(USERNAME);
        });
    });
    it('should be able to update a user programmatically', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const EMAIL = `${USERNAME}@realm.io`;
            const update = yield usernamePasswordAuthService.createOrUpdateUser(USERNAME, "password", true, { 'email': EMAIL });
            chai_1.expect(update).not.to.be.undefined;
            chai_1.expect(update.user).not.to.be.undefined;
            chai_1.expect(update.created).to.be.true;
            chai_1.expect(update.user.metadata[0]['key']).to.eq('email');
            chai_1.expect(update.user.metadata[0]['value']).to.eq(EMAIL);
        });
    });
    describe("createOrUpdateUser", () => {
        describe("programatically", () => {
            it('should be able to update a user', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const EMAIL = `${USERNAME}@realm.io`;
                    const update = yield usernamePasswordAuthService
                        .createOrUpdateUser(USERNAME, "password", true, { 'email': EMAIL });
                    chai_1.expect(update).not.to.be.undefined;
                    chai_1.expect(update.user).not.to.be.undefined;
                    chai_1.expect(update.created).to.be.true;
                    chai_1.expect(update.user.metadata[0]['key']).to.eq('email');
                    chai_1.expect(update.user.metadata[0]['value']).to.eq(EMAIL);
                });
            });
        });
        describe("via REST API", () => {
            it('with an admin token should be able to create or update a user', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const EMAIL = `${USERNAME}@realm.io`;
                    const token = new Token_1.RefreshToken({ identity: "some-admin", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
                    chai_1.assert.isFulfilled(superagent.put(`${server.url}/auth/users`)
                        .set({
                        Authorization: token,
                    })
                        .send({
                        providerId: USERNAME,
                        provider: "password",
                        isAdmin: true,
                        metadata: {
                            'email': EMAIL
                        }
                    }));
                });
            });
            it('should be able to update a user with admin refresh token', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const EMAIL = `${USERNAME}@realm.io`;
                    const create = yield usernamePasswordAuthService.createOrUpdateUser(USERNAME, "password", false, { 'email': EMAIL });
                    const token = new Token_1.RefreshToken({ identity: "some-admin", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
                    const response = yield superagent.put(`${server.url}/auth/users`)
                        .set({
                        Authorization: token,
                    })
                        .send({
                        providerId: USERNAME,
                        provider: "password",
                        isAdmin: true,
                        metadata: {
                            'email': EMAIL
                        }
                    });
                    chai_1.expect(response.body).not.to.be.undefined;
                    chai_1.expect(response.body["user"]).not.to.be.undefined;
                    chai_1.expect(response.body["created"]).not.to.be.true;
                    chai_1.expect(response.body["user"]["userId"]).not.to.eq(USERNAME);
                    chai_1.expect(response.body["user"]["accounts"]["0"].providerId).to.eq(USERNAME);
                    chai_1.expect(response.body["user"]["isAdmin"]).to.eq(true);
                });
            });
            it('should be able to update a user with user refresh token', function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const EMAIL = `${USERNAME}@realm.io`;
                    const NEWEMAIL = "new-email@realm.io";
                    const create = yield usernamePasswordAuthService.createOrUpdateUser(USERNAME, "password", false, { 'email': EMAIL });
                    const token = new Token_1.RefreshToken({ identity: create.user.userId, isAdmin: create.user.isAdmin, appId: 'io.realm.Test' }).sign(server.privateKey);
                    const response = yield superagent.put(`${server.url}/auth/users`)
                        .set({
                        Authorization: token,
                    })
                        .send({
                        providerId: USERNAME,
                        provider: "password",
                        isAdmin: false,
                        metadata: {
                            'email': NEWEMAIL
                        }
                    });
                    chai_1.expect(response.body).not.to.be.undefined;
                    chai_1.expect(response.body["user"]).not.to.be.undefined;
                    chai_1.expect(response.body["created"]).not.to.be.true;
                    chai_1.expect(response.body["user"]["userId"]).not.to.eq(USERNAME);
                    chai_1.expect(response.body["user"]["accounts"]["0"].providerId).to.eq(USERNAME);
                    chai_1.expect(response.body["user"]["metadata"]["0"].key).to.eq("email");
                    chai_1.expect(response.body["user"]["metadata"]["0"].value).to.eq(NEWEMAIL);
                    chai_1.expect(response.body["user"]["isAdmin"]).to.eq(false);
                });
            });
            it("without an admin token it should reject with an error when updating isAdmin", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const EMAIL = `${USERNAME}@realm.io`;
                const create = yield usernamePasswordAuthService.createOrUpdateUser(USERNAME, "password", false, { 'email': EMAIL });
                const token = new Token_1.RefreshToken({ identity: create.user.userId, isAdmin: create.user.isAdmin, appId: 'io.realm.Test' }).sign(server.privateKey);
                chai_1.assert.isRejected(superagent.put(`${server.url}/auth/users`)
                    .set({
                    Authorization: token,
                })
                    .send({
                    providerId: USERNAME,
                    provider: "password",
                    isAdmin: true,
                    metadata: {
                        'email': EMAIL
                    }
                }), "Forbidden");
            }));
        });
    });
    describe("changePassword", () => {
        describe("programatically", () => {
            it("should be able to change a users password", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const ORIGINALPASSWORD = 'originalpassword';
                    const NEWPASSWORD = 'newpassword';
                    const BADPASSWORD = 'badpassword';
                    const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                    yield usernamePasswordAuthService.changePassword(NEWPASSWORD, user.userId);
                    yield chai_1.assert.isFulfilled(usernamePasswordAuthService.attemptToLogin(USERNAME, NEWPASSWORD));
                });
            });
            it("should respond with a 611 error code with a bad change password request", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    const ORIGINALPASSWORD = 'originalpassword';
                    const NEWPASSWORD = 'newpassword';
                    const BADPASSWORD = 'badpassword';
                    const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                    yield usernamePasswordAuthService.changePassword(NEWPASSWORD, user.userId);
                    const promise = usernamePasswordAuthService.attemptToLogin(USERNAME, BADPASSWORD);
                    yield chai_1.assert.isRejected(promise, "The provided credentials are invalid or the user does not exist.");
                });
            });
        });
        describe("via REST API", () => {
            const ORIGINALPASSWORD = 'originalpassword';
            const NEWPASSWORD = 'newpassword';
            const BADPASSWORD = 'badpassword';
            it("without a token it should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                chai_1.assert.isRejected(superagent.put(`${server.url}/auth/password`).send({
                    newPassword: NEWPASSWORD
                }), "Forbidden");
            }));
            it("with the wrong token should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                const token = new Token_1.RefreshToken({ identity: "some user", isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isRejected(superagent.put(`${server.url}/auth/password`)
                    .set({
                    Authorization: token,
                })
                    .send({
                    newPassword: NEWPASSWORD
                }), "Bad Request");
            }));
            it("with an admin token should change the password", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                const token = new Token_1.RefreshToken({ identity: "some user", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isFulfilled(superagent.put(`${server.url}/auth/password`)
                    .set({
                    Authorization: token,
                })
                    .send({
                    userId: user.userId,
                    newPassword: NEWPASSWORD
                }));
            }));
            it("with the users token should change the password", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                const token = new Token_1.RefreshToken({ identity: user.userId, isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isFulfilled(superagent.put(`${server.url}/auth/password`)
                    .set({
                    Authorization: token,
                })
                    .send({
                    newPassword: NEWPASSWORD
                }));
            }));
            it("with the users token should not change another user's password", () => __awaiter(this, void 0, void 0, function* () {
                const USERNAME = uuid.v4();
                const user = yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
                const token = new Token_1.RefreshToken({ identity: user.userId, isAdmin: false, appId: 'io.realm.Test' }).sign(server.privateKey);
                yield chai_1.assert.isRejected(superagent.put(`${server.url}/auth/password`)
                    .set({
                    Authorization: token,
                })
                    .send({
                    userId: "some-other-user",
                    newPassword: NEWPASSWORD
                }), "Bad Request");
            }));
            it("should respond with AccessDenied, 614 error code, with a bad change password request", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const USERNAME = uuid.v4();
                    yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
                    try {
                        const response = yield superagent.put(`${server.url}/auth/password`)
                            .send({
                            newPassword: NEWPASSWORD
                        });
                    }
                    catch (err) {
                        chai_1.expect(err).to.not.be.undefined;
                        chai_1.expect(err["status"]).to.be.eq(403);
                        chai_1.expect(err["response"]["body"]["code"]).to.be.eq(614);
                    }
                });
            });
        });
    });
    it('should respond with a 601 error code with a bad create user request via REST API', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const EMAIL = `${USERNAME}@realm.io`;
            const USERID = "bad user ID";
            const token = new Token_1.RefreshToken({ identity: "some-admin", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
            try {
                const response = yield superagent.put(`${server.url}/auth/users`)
                    .send({
                    providerId: USERNAME,
                    provider: "password",
                    isAdmin: true,
                    metadata: {
                        'email': EMAIL
                    },
                    userId: USERID
                })
                    .set({
                    Authorization: token,
                });
            }
            catch (err) {
                console.log("Status" + err["status"]);
                console.log(err["response"]);
                chai_1.expect(err).to.not.be.undefined;
                chai_1.expect(err["status"]).to.be.eq(400);
                chai_1.expect(err["response"]["body"]["code"]).to.be.eq(601);
            }
        });
    });
    it('should be able to create or update a user with a userId via REST API', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const EMAIL = `${USERNAME}@realm.io`;
            const USERID = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~:/?#[]@!$&'()*+,;=`.";
            const token = new Token_1.RefreshToken({ identity: "some-admin", isAdmin: true, appId: 'io.realm.Test' }).sign(server.privateKey);
            const response = yield superagent.put(`${server.url}/auth/users`)
                .send({
                providerId: USERNAME,
                provider: "password",
                isAdmin: true,
                metadata: {
                    'email': EMAIL
                },
                userId: USERID
            })
                .set({
                Authorization: token,
            });
            chai_1.expect(response.body).not.to.be.undefined;
            chai_1.expect(response.body["user"]).not.to.be.undefined;
            chai_1.expect(response.body["created"]).to.be.true;
            chai_1.expect(response.body["user"]["userId"]).not.to.eq(USERNAME);
            chai_1.expect(response.body["user"]["userId"]).to.eq(USERID);
            chai_1.expect(response.body["user"]["accounts"]["0"].providerId).to.eq(USERNAME);
            chai_1.expect(response.body["user"]["isAdmin"]).to.eq(true);
        });
    });
});
//# sourceMappingURL=user-metadata-integration.spec.js.map