"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TestServer_1 = require("../TestServer");
const services = require("../services");
const chai_1 = require("chai");
const chai = require("chai");
chai.use(require("chai-as-promised"));
const superagent = require("superagent");
const uuid = require("uuid");
describe('Username Password HTTP Auth Integration Tests', function () {
    let server;
    let usernamePasswordService;
    before(() => __awaiter(this, void 0, void 0, function* () {
        server = new TestServer_1.TestServer();
        server.addService(new services.SyncService());
        server.addService(new services.RealmDirectoryService());
        usernamePasswordService = new services.UsernamePasswordAuthService();
        server.addService(usernamePasswordService);
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        yield server.shutdown();
    }));
    it('test user creation', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const response = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: password, register: true }));
            chai_1.assert.isDefined(response.body.refresh_token);
            chai_1.assert.isDefined(response.body.refresh_token.token_data);
            const tokenData = response.body.refresh_token.token_data;
            chai_1.assert.isFalse(tokenData.isAdmin);
        });
    });
    it('test existing user authentication', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: password, register: true }));
            const response = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: password }));
            chai_1.assert.isDefined(response.body.refresh_token);
            chai_1.assert.isDefined(response.body.refresh_token.token_data);
            const tokenData = response.body.refresh_token.token_data;
            chai_1.assert.isFalse(tokenData.isAdmin);
        });
    });
    it('test existing user bad password', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: password, provider: 'password', register: true }));
            const response = yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: 'bad password', provider: 'password' }), "Unauthorized");
            chai_1.assert.equal(response.status, 401);
        });
    });
    it(' test get realm access token', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const createResponse = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ username: username, password: password, register: true, is_admin: true }));
            chai_1.assert.isString(createResponse.body.refresh_token.token);
            const realmTokenResponse = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth/`)
                .send({ path: '/myrealm', data: createResponse.body.refresh_token.token, provider: 'realm' }));
            chai_1.assert.isString(realmTokenResponse.body.access_token.token);
        });
    });
    it('should be able to programmatically store and retrieve the same user', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield usernamePasswordService.createOrUpdateUser('testuserId', "debug", true);
            const foundUser = yield usernamePasswordService.getUserById(response.user.userId);
            chai_1.assert.isDefined(foundUser);
            chai_1.assert.equal(foundUser.userId, response.user.userId);
            chai_1.assert.isTrue(foundUser.isAdmin);
        });
    });
    it('should be able to salt and hash a clear text password', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const salt = yield usernamePasswordService.createSalt(10000);
            const hash = yield usernamePasswordService.hashPassword('goodpassword', salt);
            const passwordIsSame = yield usernamePasswordService.comparePassword('goodpassword', salt, hash);
            const passwordIsNotSame = yield usernamePasswordService.comparePassword('badpassword', salt, hash);
            chai_1.assert.isTrue(passwordIsSame);
            chai_1.assert.isFalse(passwordIsNotSame);
        });
    });
    it('should respond with error if user doesn´t exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const err = yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth/`).send({
                app_id: '',
                data: 'some-unknown-user',
                provider: 'password',
                user_info: {
                    password: 'whatever'
                }
            }), "Unauthorized");
            chai_1.assert.equal(err.status, 401);
            chai_1.assert.equal(err.response.body.code, 611);
            chai_1.assert.equal(err.response.body.status, 401);
            chai_1.assert.equal(err.response.body.title, "The provided credentials are invalid or the user does not exist.");
        });
    });
});
//# sourceMappingURL=username-password-http-auth.spec.js.map