"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const superagent = require("superagent");
const Realm = require("realm");
const uuid = require("uuid");
const URI = require("urijs");
const TestServer_1 = require("../TestServer");
const services = require(".");
const Token_1 = require("../shared/Token");
describe('RealmDirectoryService Tests', function () {
    let server;
    let username = `Manny${uuid.v4()}`;
    let password = `bat${uuid.v4()}`;
    let userToken;
    let adminUsername = `Admin${uuid.v4()}`;
    let adminPassword = `admin${uuid.v4()}`;
    let adminUserToken;
    let realmDirectoryService;
    let usernamePasswordService;
    let authServiceHandle;
    let realmsServiceHandle;
    let tmpDir;
    let authServiceUrl;
    let realmsServiceUrl;
    let serverUrl;
    beforeEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            server = new TestServer_1.TestServer();
            server.addService(new services.UsernamePasswordAuthService());
            server.addService(new services.RealmDirectoryService());
            server.addService(new services.SyncService());
            server.addService(new services.SyncProxyService());
            yield server.start();
            serverUrl = `http://${server.address}`;
            authServiceHandle = yield server.discovery.find('auth');
            realmsServiceHandle = yield server.discovery.find('realms');
            authServiceUrl = new URI(`http://${authServiceHandle.address}:${authServiceHandle.port}`)
                .segment("/auth").toString();
            realmsServiceUrl = new URI(`http://${realmsServiceHandle.address}:${realmsServiceHandle.port}`)
                .segment("/realms").toString();
            usernamePasswordService = server.services.find(s => s instanceof services.UsernamePasswordAuthService);
            realmDirectoryService = server.services.find(s => s instanceof services.RealmDirectoryService);
            let userResponse = yield superagent
                .post(authServiceUrl)
                .send({ username: username, password: password, register: true });
            userToken = userResponse.body.refresh_token.token;
            let adminUserResponse = yield superagent
                .post(authServiceUrl)
                .send({ username: adminUsername, password: adminPassword, register: true, is_admin: true });
            adminUserToken = adminUserResponse.body.refresh_token.token;
        });
    });
    afterEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            if (server) {
                yield server.shutdown();
            }
            Realm.clearTestState();
        });
    });
    it('registers realms when getting an access token', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const realmAccessTokenResponse = yield superagent
                .post(authServiceUrl)
                .send({ path: '/myrealm', data: adminUserToken, provider: 'realm' });
            chai_1.expect(realmAccessTokenResponse.body.access_token.token).to.be.a('string');
            chai_1.expect(realmDirectoryService.realm.objectForPrimaryKey('RealmFile', '/myrealm')).to.not.be.undefined;
        });
    });
    it('registers the internal realms used by ROS', function () {
        chai_1.expect(realmDirectoryService.realm.objectForPrimaryKey('RealmFile', '/__admin')).to.not.be.undefined;
        chai_1.expect(realmDirectoryService.realm.objectForPrimaryKey('RealmFile', '/__permissions')).to.not.be.undefined;
    });
    it('can properly ask for a realm with an Admin Token User access_token ', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const privateKey = server.privateKey;
            const adminToken = yield Token_1.generateAdminToken(privateKey);
            const adminTokenUser = Realm.Sync.User.adminUser(adminToken, "some url");
            const url = new URI(realmsServiceUrl)
                .segment('files')
                .segment('%2Fproducts')
                .toString();
            const response = yield superagent
                .get(url)
                .set({
                'X-Realm-Access-Token': adminTokenUser.token
            });
            chai_1.expect(response.body.path).to.be.eq('/products');
            chai_1.expect(response.body.syncLabel).to.not.be.undefined;
            chai_1.expect(realmDirectoryService.realm.objectForPrimaryKey('RealmFile', '/products')).to.not.be.undefined;
        });
    });
    it('it can ask for a realm path with ~ and be expanded with the userId', function () {
        return __awaiter(this, void 0, void 0, function* () {
            this.timeout(50000);
            yield usernamePasswordService.attemptToRegister('sally', 'ilovesushi', true);
            const user = yield Realm.Sync.User.login(serverUrl, 'sally', 'ilovesushi');
            const realm = yield Realm.open({
                sync: {
                    url: `realm://${server.address}/~/products`,
                    user: user
                }
            });
            const foundRealmFile = realmDirectoryService.realm.objectForPrimaryKey('RealmFile', `/${user.identity}/products`);
            chai_1.expect(foundRealmFile).to.not.be.undefined;
        });
    });
    it.skip('sets and gets permissions on a realm', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const realmAccessTokenResponse = yield superagent
                .post(authServiceUrl)
                .send({ path: '/myrealm', data: adminUserToken });
            yield superagent.post(`${realmsServiceUrl}/%2fmyrealm/permissions`).send({
                type: 'all',
                accessLevel: 'read'
            });
            let response = yield superagent.get(`${realmsServiceUrl}/%2fmyrealm/permissions`);
            chai_1.expect(response.body).to.be.a('object');
            chai_1.expect(response.body.path).to.eq('/myrealm');
            chai_1.expect(response.body.creatorId).to.eq(adminUsername);
            chai_1.expect(response.body.permissions).to.be.a('array');
            chai_1.expect(response.body.permissions[0].type).to.eq('all');
            chai_1.expect(response.body.permissions[0].accessLevel).to.eq('read');
            yield superagent.post(`${realmsServiceUrl}/%2fmyrealm/permissions`).send([
                {
                    type: 'all',
                    accessLevel: 'write'
                },
                {
                    type: 'user',
                    value: 'Billy',
                    accessLevel: 'admin'
                },
            ]);
            response = yield superagent.get(`${realmsServiceUrl}/%2fmyrealm`);
            chai_1.expect(response.body).to.be.a('object');
            chai_1.expect(response.body.path).to.eq('/myrealm');
            chai_1.expect(response.body.creatorId).to.eq(adminUsername);
            chai_1.expect(response.body.permissions).to.be.a('array');
            chai_1.expect(response.body.permissions.length).to.eq(2);
            chai_1.expect(response.body.permissions[0].type).to.eq('all');
            chai_1.expect(response.body.permissions[0].accessLevel).to.eq('write');
            chai_1.expect(response.body.permissions[1].type).to.eq('user');
            chai_1.expect(response.body.permissions[1].value).to.eq('Billy');
            chai_1.expect(response.body.permissions[1].accessLevel).to.eq('admin');
            yield superagent.post(`${realmsServiceUrl}/%2fmyrealm/permissions`).send({
                type: 'all',
                accessLevel: 'none'
            });
            response = yield superagent.get(`${realmsServiceUrl}/%2fmyrealm`);
            chai_1.expect(response.body.permissions.length).to.eq(1);
            chai_1.expect(response.body.permissions[0].type).to.eq('user');
            chai_1.expect(response.body.permissions[0].value).to.eq('Billy');
        });
    });
});
//# sourceMappingURL=RealmDirectoryService.spec.js.map